<?php
namespace DistributeHelper;
error_reporting(E_ALL & ~E_NOTICE);
include "Utility.php";

class PackageClass
{
	private static $config = null;     //配置信息
	//当所有包都安装完毕后
	public static function postUpdate($vendorDir,$rootPkg,$vendorPkgs)
	{
		//初始化
		if(! self::initialize($vendorDir)){
			self::setUpdateFailed('INITIALIZE_FAILED');
			return false;
		}
		
		//依赖库都已下载，默认先给个进度
		self::updateProgress(0,100);
		
		//逐个依赖仓库操作
		self::postUpdateVendorPkgs($vendorDir,$vendorPkgs);
		
		//顶层仓库操作
		self::postUpdateRootPkg($vendorDir,$rootPkg);
		
		//全部操作完毕
		self::setUpdateOK(true);
		return true;
	}
	
	private static function postUpdateVendorPkgs($vendorDir,$vendorPkgs)
	{
		foreach($vendorPkgs as $i=>$pkg){
			//获取包信息
			$repoDir = pathJoin($vendorDir,$pkg->getName());
			$packageInfo = self::getPackageInfo($vendorDir,$repoDir,$pkg);
			
			//获取当前包的module文件
			self::fetchModuleFiles($packageInfo);
			
			//获取当前包的common文件
			self::fetchCommonFiles($packageInfo);

			//拷贝当前包的config文件
			self::copyConfigFiles($packageInfo);
						
			//更新部署进度
			$percent = intval(100.0 * ($i+1) / count(vendorPkgs));
			self::updateProgress(1,$percent);
		}
	}
	private static function postUpdateRootPkg($vendorDir,$pkg)
	{
		//获取包信息
		$repoDir = dirname($vendorDir);
		$packageInfo = self::getPackageInfo($vendorDir,$repoDir,$pkg);
		
		//获取当前包的module文件
		self::fetchModuleFiles($packageInfo);
		
		//获取当前包的common文件
		self::fetchCommonFiles($packageInfo);

		//获取当前包的config文件
		self::fetchConfigFiles($packageInfo);
		
		//注册全局路径
		self::registerPath($packageInfo);
		
		//注册自启动程序
		self::registerAutorun($packageInfo);
	}
	//初始化
	private static function initialize($vendorDir)
	{
		//创建.distribute目录
		$distributeDir = pathJoin(dirname($vendorDir),'.distribute');
		if( !file_exists($distributeDir) ){
			createDirectory($distributeDir);
		}
		
		//创建module目录
		$moduleDir = pathJoin(dirname($vendorDir),'module');
		if( !file_exists($moduleDir) ){
			createDirectory($moduleDir);
		}
		
		//设置日志文件路径
		if(! defined(LOG_PATH)){
			$logPath = pathJoin($distributeDir,'log.txt');
			define('LOG_PATH',$logPath);
		}

		//设置进度文件路径
		if(! defined(PROGRESS_PATH)){
			$progressPath = pathJoin($distributeDir,'progress.txt');
			define('PROGRESS_PATH',$progressPath);
		}
		
		//获取部署配置信息
		$cfgPath = pathJoin($distributeDir,'config.json');
		if(! file_exists($cfgPath) ){
			logError("distribute config file not exists",$cfgPath);
			return false;
		}
		self::$config = loadConfig($cfgPath);
		if(!self::$config){
			logError("Failed to load distribute config",$cfgPath);
			return false;
		}
		return true;
	}
	
	private static function updateProgress($step,$percent)
	{
		file_put_contents(PROGRESS_PATH,strval($percent));
	}
	private static function setUpdateFailed($msg)
	{
		file_put_contents(PROGRESS_PATH,"FAIL:" . $msg);
	}
	private static function setUpdateOK($restart)
	{
		if($restart){
			file_put_contents(PROGRESS_PATH,"OK:NEED_RESTART");
		}
		else{
			file_put_contents(PROGRESS_PATH,"OK:NO_RESTART");
		}
	}
	//获取包相关信息
	private static function getPackageInfo($vendorDir,$repoDir,$pkg)
	{
		$config = self::$config;
		
		$cfgPath = pathJoin($repoDir,'composer.json');
		$data = json_decode(file_get_contents($cfgPath), true);
		$moduleName = $data["distribute-helper"]["module-name"];

		
		//获取安装目录
		$rootDir = dirname($vendorDir);
		$commonDir = pathJoin($rootDir,'siscom');
		$configDir = pathJoin($rootDir,'config',$moduleName);
		$moduleDir = pathJoin($rootDir,'module',$moduleName . '-' . $pkg->getPrettyVersion());
		
		//获取缓存目录
		$cacheName = str_replace('/','-',$pkg->getName()) . '-' . $pkg->getPrettyVersion();
		$cacheDir = pathJoin($config['cache-dir'],$cacheName);
		
		//返回包的相关信息
		return array(
			"repoName" => $pkg->getName(),
			"version" => $pkg->getPrettyVersion(),
			"cacheDir" => $cacheDir,
			"toolDir" => $config['tool-dir'],
			"ossHost" => $config['oss-host'],
			"moduleName" => $moduleName,
			"commonDir" => $commonDir,
			"configDir" => $configDir,
			"moduleDir" => $moduleDir,
			"helper" => $data["distribute-helper"],
		);
	}
	
	//下载包module文件
	private static function fetchModuleFiles($packageInfo)
	{
		$toolDir = $packageInfo['toolDir'];
		$cacheDir = $packageInfo['cacheDir'];
		$repoName = $packageInfo['repoName'];
		$version = $packageInfo['version'];
		$moduleDir = $packageInfo['moduleDir'];
		$ossHost = $packageInfo['ossHost'];
		
		$moduleFiles = $packageInfo['helper']['module-files'];
		if(! $moduleFiles){
			return;
		}
		
		foreach($moduleFiles as $fileInfo){
			$fileName = $fileInfo['name'];
			logInfo($repoName,"fetching module file -- ",$fileName);

			//下载文件到缓存区
			$filePath = downloadToDir($toolDir,$ossHost,$repoName,$version,$fileName,$cacheDir);
			if(! $filePath ){
				logError($repoName,"fetch module file failed",$fileName);
				return false;
			}

			// 解压文件
			if( isZipFile($fileName) ){
				if(! unzipFile($toolDir,$filePath,$moduleDir)) {
					logError($repoName,"unzip module file failed",$fileName);
					return false;
				}
			}
		}
		return true;
	}
	
	//下载包common文件
	private static function fetchCommonFiles($packageInfo)
	{
		$toolDir = $packageInfo['toolDir'];
		$cacheDir = $packageInfo['cacheDir'];
		$repoName = $packageInfo['repoName'];
		$version  = $packageInfo['version'];
		$commonDir = $packageInfo['commonDir'];
		$ossHost = $packageInfo['ossHost'];
		$commonFiles = $packageInfo['helper']['common-files'];
		if(! $commonFiles){
			return;
		}
		foreach($commonFiles as $fileInfo){
			$fileName = $fileInfo['name'];
			logInfo($repoName,"fetching common file -- ",$fileName);
			//下载文件到缓存区
			$filePath = downloadToDir($toolDir,$ossHost,$repoName,$version,$fileName,$cacheDir);
			if(! $filePath ){
				logError($repoName,"fetch common file failed",$fileName);
				continue;
			}

			// 解压文件
			if( isZipFile($fileName) ){
				if(! unzipFile($toolDir,$filePath,$commonDir)) {
					logError($repoName,"unzip common file failed",$fileName);
					return false;
				}
			}
		}
		return true;
	}
	
	//拷贝config文件
	private static function copyConfigFiles($packageInfo)
	{
		$cacheDir = $packageInfo['cacheDir'];
		$repoName = $packageInfo['repoName'];
		$version = $packageInfo['version'];
		$configDir = $packageInfo['configDir'];
		$moduleDir = $packageInfo['moduleDir'];
		$configFiles = $packageInfo['helper']['config-files'];
		if(! $configFiles){
			return;
		}
		foreach($configFiles as $fileInfo){
			$fileName = $fileInfo['name'];
			$filePath = pathJoin($moduleDir,$fileName);
			if(! file_exists($filePath) ){
				logError($repoName,"config file not exists",$filePath);
				return false;
			}
			if(!$fileInfo['override'] && file_exists($targetPath)){
				logInfo($repoName,"skip copy config file",$targetPath);
			}
			$targetPath = pathJoin($configDir,$fileName);
			fileCopy($filePath,$targetPath);
		}
		return true;
	}
	
	private static function registerPath($packageInfo)
	{
	}
	
	private static function registerAutorun($packageInfo)
	{
	}
}